@@ -19,16 +19,12 @@
 
 #include <locale.h>
 #include <libintl.h>
-#include <glib.h>
 #include <glib/gi18n.h>
-#include <hildon/hildon.h>
-#include <libosso.h>
-#include <gconf/gconf-client.h>
-#include <curl/curl.h>
 #include <string.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include "sound.h"
+#include "orientation.h"
 #include "100boxes.h"
 
 static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data);
@@ -44,17 +40,21 @@ static void show_high_score(struct game_data *d);
 static void show_error_message(struct game_data *d, const gchar *msg);
 static void save_state(gpointer data);
 static void sound_button_toggled(HildonCheckButton *button, gpointer data);
+static void on_window_is_topmost_changed(HildonProgram *prog, GParamSpec *par, gpointer data);
 
 int main (int argc, char **argv) {
 	GtkWidget *win;
-	GtkWidget *box_table;
 	GtkWidget *box[100];
-	GtkWidget *container;
-	GtkWidget *panel;
 	GtkWidget *new_button;
 	GtkWidget *score_label;
 	GtkWidget *sound_button;
 	GtkWidget *guide_button;
+	gpointer h_container;
+	gpointer v_container;
+	gpointer *panel;
+	gpointer *box_table;
+	HildonPortraitFlags flags;
+	HildonProgram *program;
 	GError *error = NULL;
 	gint i,j;
 	GSList *seq;
@@ -88,7 +88,7 @@ int main (int argc, char **argv) {
 	}
 	hildon_gtk_init(&argc, &argv);
 	osso_application_set_autosave_cb(d.osso_context, save_state, &d);
-	//g_debug("System init: done");
+	g_debug("System init: done");
 
 	// Initializie gconf and load application state and preferences
 	gconf_init(argc, argv, NULL);
@@ -97,48 +97,57 @@ int main (int argc, char **argv) {
 	s.total = gconf_client_get_int(d.client, KEY_TOTAL_SCORE, &error);
 	s.best = gconf_client_get_int(d.client, KEY_BEST_SCORE, &error);
 
+	// Listen for screen orientation changes
+	program = HILDON_PROGRAM(hildon_program_get_instance());
+	g_signal_connect(program, "notify::is-topmost", G_CALLBACK(on_window_is_topmost_changed), (gpointer)&d);
+	orientation_init(&d);
+
 	// Create the main window
 	win = hildon_window_new();
 	gtk_window_set_title(GTK_WINDOW(win), "100 Boxes");
 	d.window = (GtkWindow*) win;
 
 	// Create the box table
-	box_table = gtk_table_new(10, 10, TRUE);
-	gtk_widget_set_size_request(GTK_WIDGET(box_table), HEIGHT, -1);
+	d.box_table = gtk_table_new(10, 10, TRUE);
+	box_table = g_object_ref((gpointer)d.box_table); // keep reference to avoid deletion when detaching from container
 	for (i=0; i<10; i++) {
 		for (j=0; j<10; j++) {
 			box[10*i+j] = gtk_event_box_new();
-			gtk_table_attach_defaults(GTK_TABLE(box_table), GTK_WIDGET(box[10*i+j]), j, j+1, i, i+1);
+			gtk_table_attach_defaults(GTK_TABLE(d.box_table), GTK_WIDGET(box[10*i+j]), j, j+1, i, i+1);
 			g_signal_connect(G_OBJECT(box[10*i+j]), "button_press_event", G_CALLBACK(box_clicked), (gpointer)&d);
 		}
 	}
 
 	// Create other GUI elements
-	container = gtk_hbox_new(FALSE, PADDING);
-	panel = gtk_vbox_new(TRUE, PADDING);
-	gtk_widget_set_size_request(GTK_WIDGET(panel), WIDTH-HEIGHT-PADDING, -1);
+	d.h_container = gtk_hbox_new(FALSE, PADDING);
+	d.v_container = gtk_vbox_new(FALSE, 0);
+	d.panel = gtk_vbox_new(TRUE, PADDING/2);
+	 // Keep references to avoid deletion when detaching from containers
+	h_container = g_object_ref((gpointer)d.h_container);
+	v_container = g_object_ref((gpointer)d.v_container);
+	panel = g_object_ref((gpointer)d.panel);
 
 	new_button = gtk_button_new_with_label(_("New game"));
 	hildon_helper_set_logical_font(GTK_WIDGET(new_button), "LargeSystemFont");
 	g_signal_connect(GTK_WIDGET(new_button), "clicked", G_CALLBACK(new_clicked), (gpointer)&d);
-	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(new_button));
+	gtk_container_add(GTK_CONTAINER(d.panel), GTK_WIDGET(new_button));
 
 	score_label = gtk_label_new(_("Score: 0"));
 	hildon_helper_set_logical_font(GTK_WIDGET(score_label), "LargeSystemFont");
-	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(score_label));
+	gtk_container_add(GTK_CONTAINER(d.panel), GTK_WIDGET(score_label));
 	d.score = score_label;
 
 	sound_button = hildon_check_button_new(HILDON_SIZE_AUTO);
 	gtk_button_set_label(GTK_BUTTON(sound_button), _("Sound"));
 	s.sound_on = ! gconf_client_get_bool(d.client, KEY_MUTE, NULL);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(sound_button), s.sound_on);
-	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(sound_button));
+	gtk_container_add(GTK_CONTAINER(d.panel), GTK_WIDGET(sound_button));
 	g_signal_connect(sound_button, "toggled", G_CALLBACK(sound_button_toggled), (gpointer)&d);
 
 	guide_button = gtk_button_new_with_label(_("Guide"));
 	g_signal_connect(GTK_WIDGET(guide_button), "clicked", G_CALLBACK(guide_clicked), (gpointer)&d);
-	gtk_container_add(GTK_CONTAINER(panel), GTK_WIDGET(guide_button));
-	//g_debug("GUI creation: done");
+	gtk_container_add(GTK_CONTAINER(d.panel), GTK_WIDGET(guide_button));
+	g_debug("GUI creation: done");
 
 	// Initialize game state
 	seq = gconf_client_get_list(d.client, KEY_GAME_SEQ, GCONF_VALUE_INT, NULL);
@@ -167,27 +176,43 @@ int main (int argc, char **argv) {
 			}
 			g_slist_free(seq);
 		}
-		//g_debug("Game state restore: done");
+		g_debug("Game state restore: done");
 	} else {
 		new_game(&d);
 	}
 
 	// Assemble GUI elements
-	gtk_container_add(GTK_CONTAINER(container), GTK_WIDGET(box_table));
-	gtk_container_add(GTK_CONTAINER(container), GTK_WIDGET(panel));
-	gtk_container_add(GTK_CONTAINER(win), GTK_WIDGET(container));
+	if (d.portrait) {
+		flags = HILDON_PORTRAIT_MODE_REQUEST;
+		gtk_widget_set_size_request(GTK_WIDGET(d.box_table), HEIGHT, HEIGHT-PADDING);
+		gtk_widget_set_size_request(GTK_WIDGET(d.panel), HEIGHT, -1);
+		gtk_container_add(GTK_CONTAINER(d.v_container), GTK_WIDGET(d.panel));
+		gtk_container_add(GTK_CONTAINER(d.v_container), GTK_WIDGET(d.box_table));
+		gtk_container_add(GTK_CONTAINER(win), GTK_WIDGET(d.v_container));
+	} else {
+		flags = HILDON_PORTRAIT_MODE_SUPPORT;
+		gtk_widget_set_size_request(GTK_WIDGET(d.box_table), HEIGHT, -1);
+		gtk_widget_set_size_request(GTK_WIDGET(d.panel), WIDTH-HEIGHT-PADDING, -1);
+		gtk_container_add(GTK_CONTAINER(d.h_container), GTK_WIDGET(d.box_table));
+		gtk_container_add(GTK_CONTAINER(d.h_container), GTK_WIDGET(d.panel));
+		gtk_container_add(GTK_CONTAINER(win), GTK_WIDGET(d.h_container));
+	}
+	hildon_gtk_window_set_portrait_flags(GTK_WINDOW(win), flags);
 	g_signal_connect(win, "destroy", G_CALLBACK(gtk_main_quit), NULL);
 	gtk_widget_show_all(win);
 
 	// Main event loop
 	gtk_main();
 
-	osso_application_autosave_force(d.osso_context);
-
-	// Deinitialize OSSO
-	osso_deinitialize(d.osso_context);
+	// Release widget references, allow deletion
+	g_object_unref(h_container);
+	g_object_unref(v_container);
+	g_object_unref(panel);
+	g_object_unref(box_table);
 	
-	// Deinitialize sound
+	// Save state and deinitialize
+	osso_application_autosave_force(d.osso_context);
+	osso_deinitialize(d.osso_context);
 	clear_stock(d.sounds);
 	return 0;
 }
@@ -210,7 +235,7 @@ static void save_state(gpointer data) {
 	gconf_client_set_int(d->client, KEY_PLAYED_GAMES, d->s->played, NULL);
 	gconf_client_set_int(d->client, KEY_BEST_SCORE, d->s->best, NULL);
 	gconf_client_set_int(d->client, KEY_TOTAL_SCORE, d->s->total, NULL);
-	//g_debug("State saved");
+	g_debug("State saved");
 }
 
 static gboolean box_clicked(GtkWidget *b, GdkEventButton *event, gpointer data) {
@@ -306,7 +331,7 @@ void new_game(struct game_data *d) {
 	d->s->error = FALSE;
 	d->s->score = 0;
 	d->s->last = -1;
-	//g_debug("Starting new game");
+	g_debug("Starting new game");
 	if (d->s->sound_on) play_stock_sound(d->sounds, "START");
 	for (i=0; i<100; i++) {
 		d->s->b[i] = CLICKABLE;
@@ -464,7 +489,7 @@ static void submit_score(struct game_data *d, gchar *name) {
 	if (d->curl) {
 		// POST data: show_scores callback function will receive the reply
 		sprintf(post, "n=%s&s=%d&src=%s", name, d->s->score, SCORE_SRC);
-		//g_debug("POST:\n%s", post);
+		g_debug("POST:\n%s", post);
 		curl_easy_setopt(d->curl, CURLOPT_VERBOSE, 1);		// Set POST query
 		curl_easy_setopt(d->curl, CURLOPT_POST, 1);		// Set POST method
 		curl_easy_setopt(d->curl, CURLOPT_POSTFIELDS, post);	// Set POST data
@@ -473,7 +498,7 @@ static void submit_score(struct game_data *d, gchar *name) {
 		curl_easy_setopt(d->curl, CURLOPT_WRITEDATA, d);	// Set data passed to callback
 		res = curl_easy_perform(d->curl); 			// Do the POST
 		if (res == 0) { // success
-			//g_debug("REPLY:\n%s", d->post_reply);
+			g_debug("REPLY:\n%s", d->post_reply);
 			show_high_score(d);
 		} else {
 			show_error_message(d, _("Submission failed."));
@@ -558,7 +583,7 @@ static void show_high_score(struct game_data *d) {
 		all_area = gtk_vbox_new(FALSE, 0);
 		gtk_notebook_append_page(GTK_NOTEBOOK(tabs), GTK_WIDGET(all_area), label);
 
-		//g_debug("Getting root XML element");
+		g_debug("Getting root XML element");
 		root = xmlDocGetRootElement(doc);
 		pos = root->children;
 		pos_v = (gchar*) pos->children->content;
@@ -609,7 +634,7 @@ static void show_high_score(struct game_data *d) {
 			gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(date_l[j]), 3, 4, j+1, j+2);
 		}
 
-		//g_debug("Getting week XML element");
+		g_debug("Getting week XML element");
 		week = top->next;
 		w_pos = week->children;
 		w_pos_v = (gchar*) w_pos->children->content;
@@ -660,7 +685,7 @@ static void show_high_score(struct game_data *d) {
 			gtk_table_attach_defaults(GTK_TABLE(w_table), GTK_WIDGET(w_date_l[j]), 3, 4, j+1, j+2);
 		}
 
-		//g_debug("Getting day XML element");
+		g_debug("Getting day XML element");
 		day = week->next;
 		d_pos = day->children;
 		d_pos_v = (gchar*) d_pos->children->content;
@@ -721,3 +746,11 @@ static void show_high_score(struct game_data *d) {
 	d->post_reply = NULL;
 }
 
+static void on_window_is_topmost_changed(HildonProgram *prog, GParamSpec *par, gpointer data) {
+	if (hildon_program_get_is_topmost(prog)) {
+		orientation_enable_accelerometer(data);
+	} else {
+		orientation_disable_accelerometer(data);
+	}
+}
+
